qwebirc.ui.HilightController = new Class({
  initialize: function(session) {
    this.session = session;
    this.regex = null;
    this.prevnick = null;
  },
  match: function(text) {
    var nick = this.session.irc.nickname;
    if(nick != this.prevnick) {
      var classes = '[\\s\\.,;:]';
      this.regex = new RegExp('(^|' + classes + ')' + RegExp.escape(nick) + '(' + classes + '|$)', "i");
    }
    if(text.match(this.regex))
      return true;
    return false;
  }
});

qwebirc.ui.Beeper = new Class({
  initialize: function(session) {
    this.session = session;

    this.soundInited = false;
    this.soundReady = false;

    if(conf.ui.beep_on_mention)
      this.soundInit();
  },
  soundInit: function() {
    if(this.soundInited)
      return;
    if(!$defined(Browser.Plugins.Flash) || Browser.Plugins.Flash.version < 8)
      return;
    this.soundInited = true;

    this.soundPlayer = new qwebirc.sound.SoundPlayer(this.session);
    this.soundPlayer.addEvent("ready", function() {
      this.soundReady = true;
    }.bind(this));

    this.soundPlayer.go();
  },
  beep: function() {
    if(!this.soundReady || !conf.ui.beep_on_mention)
      return;

    this.soundPlayer.beep();
  }
});

qwebirc.ui.Flasher = new Class({
  initialize: function(session) {
    this.session = session;

    this.windowFocused = false;
    this.canUpdateTitle = true;
    this.titleText = document.title;

    var favIcon = this._getFavIcon();
    if($defined(favIcon)) {
      this.favIcon = favIcon;
      this.favIconParent = favIcon.parentNode;
      this.favIconVisible = true;
      this.emptyFavIcon = new Element("link");
      this.emptyFavIcon.rel = "shortcut icon";
      this.emptyFavIcon.href = conf.frontend.static_base_url + "images/empty_favicon.ico";
      this.emptyFavIcon.type = "image/x-icon";
      this.flashing = false;

      this.canFlash = true;
      document.addEvent("mousedown", this.cancelFlash.bind(this));
      document.addEvent("keydown", this.cancelFlash.bind(this));
    } else {
      this.canFlash = false;
    }
  },
  _getFavIcon: function() {
    var favIcons = $$("head link");
    for(var i=0;i<favIcons.length;i++)
      if(favIcons[i].getAttribute("rel") == "shortcut icon")
        return favIcons[i];
  },
  flash: function() {
    if(!conf.ui.flash_on_mention || this.windowFocused || !this.canFlash || this.flashing)
      return;

    this.titleText = document.title; /* just in case */
    var flashA = function() {
      this.hideFavIcon();
      this.canUpdateTitle = false;
      document.title = "Activity!";

      this.flasher = flashB.delay(500);
    }.bind(this);

    var flashB = function() {
      this.showFavIcon();
      this.canUpdateTitle = true;
      document.title = this.titleText;

      this.flasher = flashA.delay(500);
    }.bind(this);

    this.flashing = true;
    flashA();
  },
  cancelFlash: function() {
    if(!this.canFlash || !$defined(this.flasher))
      return;

    this.flashing = false;

    $clear(this.flasher);
    this.flasher = null;

    this.showFavIcon();
    document.title = this.titleText;
    this.canUpdateTitle = true;
  },
  hideFavIcon: function() {
    if(this.favIconVisible) {
      /* only seems to work in firefox */
      this.favIconVisible = false;
      this.favIconParent.removeChild(this.favIcon);
      this.favIconParent.appendChild(this.emptyFavIcon);
    }
  },
  showFavIcon: function() {
    if(!this.favIconVisible) {
      this.favIconVisible = true;
      this.favIconParent.removeChild(this.emptyFavIcon);
      this.favIconParent.appendChild(this.favIcon);
    }
  },
  updateTitle: function(text) {
    this.titleText = text;
    return this.canUpdateTitle;
  },
  focusChange: function(value) {
    this.windowFocused = value;

    if(value)
      this.cancelFlash();
  }
});
